package com.xhkjedu.sexam.service.report;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sexam.mapper.paperstudent.EPaperStudentMapper;
import com.xhkjedu.sexam.mapper.report.EReportSchoolMapper;
import com.xhkjedu.sexam.mapper.reportclass.ERclassMapper;
import com.xhkjedu.sexam.mapper.reportclass.ERclassRankMapper;
import com.xhkjedu.sexam.mapper.reportclass.EReportClassMapper;
import com.xhkjedu.sexam.mapper.reportstu.ERstudentMapper;
import com.xhkjedu.sexam.mapper.system.UserMapper;
import com.xhkjedu.sexam.model.reportstu.ERstudent;
import com.xhkjedu.sexam.vo.report.ERPaperVo;
import com.xhkjedu.sexam.vo.report.ERankParamVo;
import com.xhkjedu.sexam.vo.system.UserVo;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ywx
 * @className EReportClassService
 * @description 班级报告
 * @date 2022/8/8 9:46
 **/
@Service
public class EReportClassService {
    @Resource
    private EReportClassMapper eReportClassMapper;
    @Resource
    private ERstudentMapper eRstudentMapper;
    @Resource
    private ERclassMapper eRclassMapper;
    @Resource
    private ERclassRankMapper eRclassRankMapper;
    @Resource
    private EReportGenerateService eReportGenerateService;
    @Resource
    private EReportSchoolMapper eReportSchoolMapper;
    @Resource
    private EPaperStudentMapper ePaperStudentMapper;
    @Resource
    private UserMapper userMapper;

    //考试概况-学科成绩
    public List<Map> listXkCj(Integer examid, Integer classid) {
        return eRclassMapper.listByExamAndClassId(examid, classid);
    }

    //成绩单
    public List<Map> listCjd(ERstudent stu) {
        return eRstudentMapper.listByExamId(stu);
    }

    //成绩分析-搜索
    public List<Map> listCjFxForSearch(ERankParamVo rank) {
        return eRclassRankMapper.listByExamAndClassId(rank);
    }

    //成绩分析-设置
    @Transactional(rollbackFor = Exception.class)
    public List<Map> listCjFxForSet(ERankParamVo rank) {
        Integer examid = rank.getExamid();
        String subjectid = rank.getSubjectid();
        Integer createid = rank.getCreateid();
        Integer ranktype = rank.getRanktype();
        eReportSchoolMapper.delByExamAndSubjectId(examid, subjectid, ranktype);//删除考试等级报告
        List<ERPaperVo> papers = eReportSchoolMapper.listByExamAndSubjectId(examid, subjectid);//考试试卷
        List<ERstudent> students = ePaperStudentMapper.listByExamAndSubjectId(examid, subjectid);//考试试卷学生
        List<Integer> userids = students.stream().map(ERstudent::getStudentid).distinct().collect(Collectors.toList());
        List<UserVo> users = userMapper.listNameByUserIds(userids);
        Map<Integer, String> userMap = users.stream().collect(Collectors.toMap(UserVo::getUserid, UserVo::getUsername));
        eReportGenerateService.setClassGradeRank(examid, papers, students, rank.getRranks(), createid, userMap);//生成考试等级报告
        return eRclassRankMapper.listByExamId(rank);//考试等级报告列表
    }

    //单科概况
    public Map getDkGk(ERankParamVo rank) {
        Map map = eRclassMapper.findById(rank);//班级概况
        List<Map> pjfList = eRclassMapper.listPjf(rank);//平均分
        map.put("pjfList", pjfList);
        List<Map> cjdjList = eRclassMapper.listCjDj(rank);//成绩等级分布
        map.put("cjdjList", cjdjList);
        rank.setRanktype(1);//大幅进步
        List<Map> gzxsList = eRclassMapper.listGzXs(rank);//需关注学生
        map.put("gzxsList", gzxsList);
        List<Map> gpctList = eRclassMapper.listGpCt(rank);//高频错题
        for (Map ct : gpctList) {
            //高频错误项
            Integer ctype = (Integer) ct.get("ctype");
            String gpcwx = "";
            if (N_Utils.isObjectiveQuestion(ctype)) {
                String answerjson = ct.get("answerjson").toString();//学生作答情况
                String answer = JSON.parseArray(ct.get("answer").toString(), String.class).get(0);//正确答案
                Map amap = JSON.parseObject(answerjson, Map.class);
                List<Map<String, Integer>> list = new ArrayList<>();
                Integer cwnum = 0;//作答错误学生数量
                for (Object o : amap.keySet()) {
                    List<Map> o1 = (List<Map>) amap.get(o);
                    int answernum = o1.size();//作答学生数量
                    String xx = o.toString();//选项
                    if (!answer.equals(xx) && cwnum.compareTo(answernum) == -1) {
                        cwnum = answernum;
                        gpcwx = xx;
                    }
                }
            }
            ct.put("gpcwx", gpcwx);

            //知识点
            setQuestionPoint(ct);

            ct.remove("answerjson");
            ct.remove("ctype");
            ct.remove("answer");
            ct.remove("pointjson");
        }
        map.put("gpctList", gpctList);
        return map;
    }

    //试题知识点
    private void setQuestionPoint(Map ct) {
        Object pointjson = ct.get("pointjson");
        List<String> pointnames = new ArrayList<>();
        if (N_Utils.isNotEmpty(pointjson)) {
            List<Map> list = JSON.parseArray(pointjson.toString(), Map.class);
            Integer eptqid = (Integer) ct.get("eptqid");
            for (Map p : list) {
                String ques = p.get("ques").toString();
                String pointname = p.get("pointname").toString();
                List<Map> qlist = JSON.parseArray(ques, Map.class);
                for (Map q : qlist) {
                    List<Integer> eptqids = JSON.parseArray(q.get("eptqids").toString(), Integer.class);
                    if (N_Utils.numInList(eptqid, eptqids)) {
                        pointnames.add(pointname);
                        break;
                    }
                }
            }
        }
        ct.put("pointnames", pointnames);
    }

    //单科概况-需关注学生
    public List<Map> listGzXs(ERankParamVo rank) {
        return eRclassMapper.listGzXs(rank);
    }

    //试卷概况
    public List<Map> listSjGk(ERankParamVo rank) {
        Map paper = eReportClassMapper.getPaperByExamSubjectId(rank);
        Integer ptype = (Integer) paper.get("ptype");
        if (ptype == 1) {
            List<Map> list = eReportClassMapper.listTkSjGk(rank);
            for (Map q : list) {
                setQuestionPoint(q);//试题知识点
            }
            return list;
        } else {
            String examname = paper.get("examname").toString();
            List<Map> list = eReportClassMapper.listFjSjGk(rank);
            for (Map q : list) {
                q.put("qstem", examname + "-第" + q.get("qorder") + "题");
            }
            return list;
        }
    }

    //试卷分析
    public Map getSjFx(ERankParamVo rank) {
        Map map = new HashMap();
        List<Map> types = eReportClassMapper.listType(rank);//大题分析
        List<Map> questions = eReportClassMapper.listQuestion(rank);//小题分析
        List<Map> points = eReportClassMapper.listPoint(rank);//知识点分析
        map.put("types", types);
        map.put("questions", questions);
        map.put("points", points);
        return map;
    }
}
