package com.xhkjedu.sexam.service.report;

import com.xhkjedu.sexam.mapper.paperstudent.EPaperStudentMapper;
import com.xhkjedu.sexam.mapper.report.ERbaseMapper;
import com.xhkjedu.sexam.mapper.report.EReportSchoolMapper;
import com.xhkjedu.sexam.mapper.report.ERsubjectMapper;
import com.xhkjedu.sexam.mapper.reportclass.ERclassMapper;
import com.xhkjedu.sexam.mapper.reportclass.ERclassRankMapper;
import com.xhkjedu.sexam.mapper.reportstu.ERstudentMapper;
import com.xhkjedu.sexam.mapper.system.UserMapper;
import com.xhkjedu.sexam.model.reportstu.ERstudent;
import com.xhkjedu.sexam.model.reportstu.ERstudentAction;
import com.xhkjedu.sexam.vo.report.ERPaperVo;
import com.xhkjedu.sexam.vo.report.ERankParamVo;
import com.xhkjedu.sexam.vo.system.UserVo;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ywx
 * @className EReportSchoolService
 * @description 校级报告
 * @date 2022/8/8 9:46
 **/
@Service
public class EReportSchoolService {
    @Resource
    private ERbaseMapper eRbaseMapper;
    @Resource
    private ERsubjectMapper eRsubjectMapper;
    @Resource
    private ERstudentMapper eRstudentMapper;
    @Resource
    private ERclassMapper eRclassMapper;
    @Resource
    private ERclassRankMapper eRclassRankMapper;
    @Resource
    private EReportGenerateService eReportGenerateService;
    @Resource
    private EReportSchoolMapper eReportSchoolMapper;
    @Resource
    private EPaperStudentMapper ePaperStudentMapper;
    @Resource
    private UserMapper userMapper;

    //考试概况
    public Map getKsGk(Integer examid) {
        Map map = eRbaseMapper.findByExamId(examid);
        //成绩概况
        List<Map> subjects = eRsubjectMapper.listByExamId(examid);
        map.put("subjects", subjects);
        return map;
    }

    //成绩单
    public List<Map> listCjd(ERstudent stu) {
        return eRstudentMapper.listByExamId(stu);
    }

    //成绩分析
    public Map listCjFx(ERankParamVo rank) {
        Map map = new HashMap();
        List<Map> classes = eRclassMapper.listByExamId(rank);
        List<Map> list = eRclassRankMapper.listByExamId(rank);
        List<Map> djList = list.stream().filter(l -> "1".equals(l.get("ranktype").toString())).collect(Collectors.toList());
        List<Map> fsdList = list.stream().filter(l -> "2".equals(l.get("ranktype").toString())).collect(Collectors.toList());
        List<Map> ljfList = list.stream().filter(l -> "3".equals(l.get("ranktype").toString())).collect(Collectors.toList());
        List<Map> ljmcList = list.stream().filter(l -> "4".equals(l.get("ranktype").toString())).collect(Collectors.toList());
        List<Map> yxsList = list.stream().filter(l -> "5".equals(l.get("ranktype").toString())).collect(Collectors.toList());
        List<Map> xksList = list.stream().filter(l -> "6".equals(l.get("ranktype").toString())).collect(Collectors.toList());
        map.put("classes", classes);
        map.put("djList", djList);
        map.put("fsdList", fsdList);
        map.put("ljfList", ljfList);
        map.put("ljmcList", ljmcList);
        map.put("yxsList", yxsList);
        map.put("xksList", xksList);
        return map;
    }

    //成绩分析-搜索
    public List<Map> listCjFxForSearch(ERankParamVo rank) {
        return eRclassRankMapper.listByExamId(rank);
    }

    //成绩分析-设置
    @Transactional(rollbackFor = Exception.class)
    public List<Map> listCjFxForSet(ERankParamVo rank) {
        Integer examid = rank.getExamid();
        String subjectid = rank.getSubjectid();
        Integer createid = rank.getCreateid();
        Integer ranktype = rank.getRanktype();
        eReportSchoolMapper.delByExamAndSubjectId(examid, subjectid, ranktype);//删除考试等级报告
        List<ERPaperVo> papers = eReportSchoolMapper.listByExamAndSubjectId(examid, subjectid);//考试试卷
        List<ERstudent> students = ePaperStudentMapper.listByExamAndSubjectId(examid, subjectid);//考试试卷学生
        List<Integer> userids = students.stream().map(ERstudent::getStudentid).distinct().collect(Collectors.toList());
        List<UserVo> users = userMapper.listNameByUserIds(userids);
        Map<Integer, String> userMap = users.stream().collect(Collectors.toMap(UserVo::getUserid, UserVo::getUsername));
        eReportGenerateService.setClassGradeRank(examid, papers, students, rank.getRranks(), createid, userMap);//生成考试等级报告
        return eRclassRankMapper.listByExamId(rank);//考试等级报告列表
    }

    //行为分析-班级
    public List<Map> listXwBj(ERstudentAction rank) {
        return eReportSchoolMapper.listXwBj(rank);
    }

    //行为分析-学生
    public List<Map> listXwXs(ERstudentAction rank) {
        return eReportSchoolMapper.listXwXs(rank);
    }

    //违纪行为分析-详情
    public List<Map> listWjXq(ERstudentAction rank) {
        return eReportSchoolMapper.listWjXq(rank);
    }
}
